#!BPY

#"""
#Name: 'Align View to Selected'
#Blender: 243
#Group: 'Object'
#Tooltip: 'Align view to selected object'
#"""
		
####---Header------------------------------------------------------------------
__author__ = "Witold.Jaworski"
__url__ = ("http://www.blender.org","http://airplanes3d.net")
__email__ = ("me@wjaworski.pl")
__version__ = "1.00"
__bpydoc__ = """\
Use this script to align view to any of its 6 sides (left, right, front, ...).
There is a standard Blender command - View\Align selected, but it always place 
the viewer perpendicular to object's XY plane. This script allows you to choose 
the side of selected object.
"""
####imports-----------------------------------------------------------------
from Blender import *
####constants---------------------------------------------------------------
MENU = """Object plane:%t
|Top (XY)%x1
|Bottom (YX)%x2
|Front (XZ)%x11
|Rear (ZX)%x12
|Left (YZ)%x21
|Right (ZY)%x22
"""
#menu option names (see the %xs from MENU-just to make the code more readable):
TOP	 = 1
BOTTOM  = 2
FRONT   = 11
REAR	= 12
LEFT	= 21
RIGHT   = 22

####module functions-----------------------------------------------------------
def ChangeViewTo(obj, option):
	"""Changes current view
		
		Arguments:
			obj:	selected object 
			option: code for the plane (int), selected from popup menu by user
	"""
	#rotationPart is returned wrong, when one of X,Y,Z scale components are negative
	#or just different. As workaround, we have to "normalize" its scale, for a moment:
	scale = obj.getSize()
	
	obj.setSize(1.0,1.0,1.0)
	
	#safely obtain the rotation part:	
	rot = obj.getMatrix("worldspace").rotationPart()
	
	#.. and, eventually, restore the non uniform scale:
	obj.setSize(scale)
	
	#axes of additional rotation:
	x = Mathutils.Vector(1,0,0)*rot #object's local x axis
	rx = Mathutils.RotationMatrix(-90,3,"r",x) #a rotation: a quarter around x
	z = Mathutils.Vector(0,0,1)*rot #object's local z axis
	rz = Mathutils.RotationMatrix(-90,3,"r",z) #a rotation: a quarter around z
	
	matrix = rot.invert() #rotation matrix for the view
						  #we will add to it additional rotations, when needed

	if option == TOP:
		pass #object matrix is oriented on this plane - no change needed
	elif option == BOTTOM:
		matrix = rx*rx*matrix
	elif option == FRONT:
		matrix = rx*matrix
	elif option == REAR:
		matrix = rz*rz*rx*matrix
	elif option == LEFT:
		matrix = rz*rx*matrix
	elif option == RIGHT:
		matrix = rz*rz*rz*rx*matrix
		
	Window.SetViewQuat(matrix.toQuat())
	Window.RedrawAll()
	
def main():
	"""main procedure of this script
	
		It handles the interaction with the user
	"""
	if not Object.GetSelected():
		Draw.PupMenu("Cannot align view:%t|There is no object selected")
	else:
		option = Draw.PupMenu(MENU)
		if option > 0 : ChangeViewTo(Object.GetSelected()[0],option)
		
#let's run the whole thing:
main()
